/* strongSwan IPsec starter comparison functions
 * Copyright (C) 2001-2002 Mathieu Lafon - Arkoon Network Security
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  See <http://www.fsf.org/copyleft/gpl.txt>.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * RCSID $Id: cmp.c,v 1.8 2006/01/06 20:24:41 as Exp $
 */

#include <string.h>

#include <freeswan.h>

#include "../pluto/constants.h"
#include "../pluto/defs.h"

#include "confread.h"
#include "interfaces.h"
#include "cmp.h"

#define struneq(a,b) (a)?((b)?(strcmp(a,b)):TRUE):(b != NULL)
#define STRCMP(obj) if (struneq(c1->obj, c2->obj)) return FALSE
#define VARCMP(obj) if (c1->obj != c2->obj) return FALSE
#define ADDCMP(obj) if (!sameaddr(&c1->obj,&c2->obj)) return FALSE
#define SUBCMP(obj) if (!samesubnet(&c1->obj,&c2->obj)) return FALSE
#define LSTCMP(obj) if (!cmp_list(c1->obj,c2->obj)) return FALSE

static bool
cmp_list(char **l1, char **l2)
{
    if ((!l1) && (!l2))
	return TRUE;
    if ((!l1) || (!l2))
	return FALSE;
    for ( ; *l1 && *l2; l1++, l2++)
    {
	if (strcmp(*l1,*l2))
	    return FALSE;
    }
    if ((*l1) || (*l2))
	return FALSE;
    return TRUE;
}

static bool
starter_cmp_end(starter_end_t *c1, starter_end_t *c2)
{
    if ((!c1) || (!c2))
	return FALSE;
    STRCMP(id);
    STRCMP(rsakey);
    STRCMP(cert);
    STRCMP(ca);
    STRCMP(groups);
    STRCMP(iface);
    ADDCMP(addr);
    ADDCMP(nexthop);
    ADDCMP(srcip);
    SUBCMP(subnet);
    VARCMP(has_client);
    VARCMP(has_client_wildcard);
    VARCMP(has_port_wildcard);
    VARCMP(has_srcip);
    VARCMP(modecfg);
    VARCMP(sendcert);
    STRCMP(updown);
    VARCMP(port);
    VARCMP(protocol);
#ifdef VIRTUAL_IP
    STRCMP(virt);
#endif
    return TRUE;
}

bool
starter_cmp_conn(starter_conn_t *c1, starter_conn_t *c2)
{
    if ((!c1) || (!c2))
	return FALSE;
    STRCMP(name);
    VARCMP(policy);
    VARCMP(sa_ike_life_seconds);
    VARCMP(sa_ipsec_life_seconds);
    VARCMP(sa_rekey_margin);
    VARCMP(sa_rekey_fuzz);
    VARCMP(sa_keying_tries);
    VARCMP(addr_family);
    VARCMP(tunnel_addr_family);
    if (!starter_cmp_end(&c1->left, &c2->left))
	return FALSE;
    if (!starter_cmp_end(&c1->right, &c2->right))
	return FALSE;
    VARCMP(startup);
    STRCMP(esp);
    STRCMP(ike);
    STRCMP(pfsgroup);
    VARCMP(dpd_delay);
    VARCMP(dpd_timeout);
    VARCMP(dpd_action);
    return TRUE;
}

bool
starter_cmp_ca(starter_ca_t *c1, starter_ca_t *c2)
{
    if ((!c1) || (!c2))
	return FALSE;
    STRCMP(name);
    STRCMP(cacert);
    STRCMP(ldaphost);
    STRCMP(crluri);
    STRCMP(crluri2);
    STRCMP(ocspuri);
    VARCMP(strict);
    VARCMP(startup);
    return TRUE;
}

bool
starter_cmp_klips(starter_config_t *c1, starter_config_t *c2)
{
    if ((!c1) || (!c2))
	return FALSE;
    LSTCMP(setup.klipsdebug);
    VARCMP(setup.fragicmp);
    STRCMP(setup.packetdefault);
    VARCMP(setup.hidetos);
    return TRUE;
}

bool
starter_cmp_pluto(starter_config_t *c1, starter_config_t *c2)
{
    if ((!c1) || (!c2))
	return FALSE;
    LSTCMP(setup.plutodebug);
    STRCMP(setup.prepluto);
    STRCMP(setup.postpluto);
    VARCMP(setup.uniqueids);
    VARCMP(setup.crlcheckinterval);
    VARCMP(setup.cachecrls);
    VARCMP(setup.strictcrlpolicy);
    VARCMP(setup.nocrsend);
    VARCMP(setup.nat_traversal);
    VARCMP(setup.keep_alive);
    STRCMP(setup.pkcs11module);
    VARCMP(setup.pkcs11keepstate);
    VARCMP(setup.pkcs11proxy);

#ifdef VIRTUAL_IP
    STRCMP(setup.virtual_private);
#endif
    return TRUE;
}

bool
starter_cmp_defaultroute(defaultroute_t *d1, defaultroute_t *d2)
{
    if ((!d1) || (!d2))
	return FALSE;
    return memcmp(d1, d2, sizeof(defaultroute_t)) == 0;
}
